<?php

namespace Barn2\Plugin\WC_Lead_Time\Integration;

use Barn2\WLT_Lib\Registerable,
	Barn2\WLT_Lib\Service;

defined( 'ABSPATH' ) || exit;

/**
 * Theme-specific overrides for WooCommerce Lead Time
 *
 * @package   Barn2/woocommerce-lead-time
 * @author    Barn2 Plugins <support@barn2.com>
 * @license   GPL-3.0
 * @copyright Barn2 Media Ltd
 */
class Theme_Compat implements Registerable, Service {

	public function register() {

		add_filter( 'body_class', [ $this, 'shop_page_classes' ] );
		add_action( 'wp_enqueue_scripts', [ $this, 'add_shop_page_spacing' ] );

		// These hooks need to be fired within init so that they can be filtered out by themes.
		add_action( 'init', function() {
			/**
			 * Filter: allows developers to disable the escaping and sanitization of the stock status html.
			 * While this may be required for the majority of themes, some themes & plugins may
			 * need to add custom functionality with markup that is stripped out by wp_kses_post.
			 *
			 * @param bool $enabled
			 * @return bool
			 */
			$enabled = apply_filters( 'wclt_theme_compatibility_stock_escape', true );

			if ( ! $enabled ) {
				return;
			}

			$theme = \wp_get_theme();

			if ( $theme->template === 'porto' ) {
				\add_filter( 'porto_woocommerce_stock_html', [ $this, 'decode_htmlspecialchars_and_sanitize' ], 100, 1 );
			}

			// general catch-all for other themes which esc_html on the stock html (e.g. Astra) or availability
			\add_filter( 'woocommerce_get_stock_html', [ $this, 'decode_htmlspecialchars_and_sanitize' ], 100, 1 );
			\add_filter( 'woocommerce_get_availability_text', [ $this, 'decode_htmlspecialchars_and_sanitize' ], 100, 1 );
		} );
	}

	public function decode_htmlspecialchars_and_sanitize( $html ) {
		return \wp_kses_post( \htmlspecialchars_decode( $html ) );
	}

	/**
	 * Add a new body class when the shop page display option is enabled.
	 *
	 * @param array $classes
	 * @return array
	 */
	public function shop_page_classes( $classes ) {

		$display = get_option( 'wclt_display_on_shop', 'no' );

		if ( $display === 'yes' && ( is_shop() || is_product_category() ) ) {
			return array_merge( $classes, [ 'wclt-display' ] );
		}

		return $classes;

	}

	/**
	 * Enqueue custom inline css to add spacing between the lead time and the product details/card.
	 * This is enqueue for specific themes only.
	 *
	 * wp_add_inline_style only works when attached to an existing stylesheet handle,
	 * so we create a dummy handle and then attach the css to the dummy handle.
	 * 
	 * @return void
	 */
	public function add_shop_page_spacing() {
		$theme = \wp_get_theme();
		$template = $theme->template;
		$themes = [ 'storefront', 'woodmart', 'jupiterx' ];

		// Load css only for the above specified themes.
		if ( ! in_array( $template, $themes, true ) ) {
			return;
		}

		wp_register_style( 'wclt-dummy-handle', false );
		wp_enqueue_style( 'wclt-dummy-handle' );

		$custom_css = ".wclt-display span.wclt_lead_time { display: inline-block; margin-top:1rem} .wclt-display p.stock.wclt_lead_time { display:block; margin-top:1rem; }";

		wp_add_inline_style( 'wclt-dummy-handle', $custom_css );
	}

}
